//+------------------------------------------------------------------+
//|                                  Kaufman Adaptive Moving Average |
//|                                            Kaufman AMA basic.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      ""

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    RsiPeriod       = 14;
extern int    AMAPeriod       = 10;
extern int    AMAPrice        = PRICE_CLOSE;
extern int    Nfast           = 2;
extern int    Nslow           = 30;
extern int    PriceFilter     = 5;
extern int    PriceFilterMode = MODE_EMA;

//
//
//
//
//

double kAMAbuffer[];
double rsi[];
double rsida[];
double rsidb[];
double slope[];
string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,rsi);
   SetIndexBuffer(1,rsida);
   SetIndexBuffer(2,rsidb);
   SetIndexBuffer(3,slope);
   SetIndexBuffer(4,kAMAbuffer);
   
   //
   //
   //
   //
   //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame == "returnBars"; if (returnBars) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+" RSI of Kaufman AMA ("+AMAPeriod+")");
   return(0);
}
int deinit()
{
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { rsi[0] = limit+1; return(0); }
         if (timeFrame != Period())
         {
            limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
            if (slope[limit]==-1) CleanPoint(limit,rsida,rsidb);
            for(int i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,timeFrame,Time[i]);               
                  rsi[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",RsiPeriod,AMAPeriod,AMAPrice,Nfast,Nslow,PriceFilter,PriceFilterMode,0,y);
                  slope[i] = iCustom(NULL,timeFrame,indicatorFileName,"",RsiPeriod,AMAPeriod,AMAPrice,Nfast,Nslow,PriceFilter,PriceFilterMode,3,y);
                  rsida[i] = EMPTY_VALUE;
                  rsidb[i] = EMPTY_VALUE;
                  if (slope[i]==-1) PlotPoint(i,rsida,rsidb,rsi);
            }
            return(0);
         }

   //
   //
   //
   //
   //
   
   if (slope[limit]==-1) CleanPoint(limit,rsida,rsidb);
   for(i=limit; i>=0; i--) kAMAbuffer[i] = iKama(iMA(NULL,0,PriceFilter,0,PriceFilterMode,AMAPrice,i),AMAPeriod,Nfast,Nslow,i);
   for(i=limit; i>=0; i--)
   {
      rsi[i]   = iRSIOnArray(kAMAbuffer,0,RsiPeriod,i);
      rsida[i] = EMPTY_VALUE;
      rsidb[i] = EMPTY_VALUE;
      slope[i] = slope[i+1];
         if (rsi[i]>rsi[i+1]) slope[i] =  1;
         if (rsi[i]<rsi[i+1]) slope[i] = -1;
         if (slope[i]==-1) PlotPoint(i,rsida,rsidb,rsi);
   }      
   return(0);
}
  
//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

double work[][3];
#define _price 0
#define _ama   1
#define _diff  2

double iKama(double price, int period, double fast, double slow, int i, double gCoeff = 2, int s=0)
{
   if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars); int r = Bars-i-1; s *= 3;
      work[r][s+_price] = price;
      if (r<period)
      {
         work[r][s+_diff] = 0;
         work[r][s+_ama]  = price;
            return(work[r][s+_ama]);          
      }
   
   //
   //
   //
   //
   //
   
   double noise   = 0;
   double efratio = 1.00;
   double fastend          = (2.0 /(fast + 1.0));
   double slowend          = (2.0 /(slow + 1.0));
   double signal           = MathAbs(work[r][s+_price]-work[r-period][s+_price]);
          work[r][s+_diff] = MathAbs(work[r][s+_price]-work[r-1][s+_price]);
          for (int k=0; k<period && (r-k)>=0;k++)
                     noise += work[r-k][s+_diff];

          //
          //
          //
          //
          //

          if (noise != 0) efratio = signal/noise;
                   double smooth  = MathPow(efratio*(fastend-slowend)+slowend,gCoeff);
                          work[r][s+_ama] = work[r-1][s+_ama] + smooth*(work[r][s+_price]-work[r-1][s+_ama]);
   return(work[r][s+_ama]);          
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}